package com.sd.business.service.board.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.board.dto.DailyBoardSelectDto;
import com.sd.business.entity.board.vo.*;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.outbound.po.OutboundOrder;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.service.board.DailyBoardService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.outbound.OutboundOrderService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.Comparator.comparing;

@Service
public class DailyBoardServiceImpl implements DailyBoardService {

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private InOutStorageBomService inOutStorageBomService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private OutboundOrderService outboundOrderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StatementOfAccountService statementOfAccountService;

    @Override
    public List<DailyBoardOutStorageBomVo> getProductionOutStorageList(DailyBoardSelectDto dto) {
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());
        List<InOutStorage> inOutStorageList = inOutStorageService.list(q -> q
                .eq(InOutStorage::getType, InOutTypeEnum.OUT.getKey())
                .eq(InOutStorage::getDetailType, OutDetailTypeEnum.PRODUCTION.getKey())
                .between(BasePo::getCreateTime, beginDate, endDate));
        if (ObjectUtil.isEmpty(inOutStorageList)) {
            return Collections.emptyList();
        }
        List<Long> inOutStorageIds = inOutStorageList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询今日出库bom数据
        List<InOutStorageBom> outStorageBomList = inOutStorageBomService.list(q -> q.in(InOutStorageBom::getInOutStorageId, inOutStorageIds));
        Map<Long, InOutStorageBom> map = outStorageBomList.stream()
                .sorted(comparing(InOutStorageBom::getBomSpecId))
                .collect(Collectors.toMap(
                        InOutStorageBom::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return v1;
                        }
                ));

        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(map.keySet());
        // 筛选出主材主库信息
        List<DailyBoardOutStorageBomVo> list = map.values().stream()
                .filter(item -> {
                    BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    return bomSpecBo != null && ObjectUtil.equals(bomSpecBo.getClassifyParentId(), 1L);
                })
                .map(item -> {
                    BomSpecBo bomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    DailyBoardOutStorageBomVo outStorageVo = new DailyBoardOutStorageBomVo();
                    outStorageVo.setBomSpecName(bomSpecBo.getBomSpecName());
                    outStorageVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
                    outStorageVo.setQuantity(item.getQuantity());
                    return outStorageVo;
                }).collect(Collectors.toList());
        return list;
    }

    @Override
    public List<OrderBomDifferenceVo> getOrderBomDifferenceList(DailyBoardSelectDto dto) {
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());
        // 差异Map
        Map<Long, OrderBomDifferenceVo> differenceMap = new HashMap<>();
        // 查询佰卓和实业的事业部id
        List<Department> departmentList = departmentService.list(q -> q.like(Department::getName, "实业").or().eq(Department::getName, "佰卓").select(BaseIdPo::getId));
        List<Long> departmentIds = departmentList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询今日生成完成任务
        Map<Long, List<ProductionWorkOrder>> taskMap = productionWorkOrderService.mapKGroup(ProductionWorkOrder::getOrderId, q -> q.between(ProductionWorkOrder::getCompleteTime, beginDate, endDate));

        // 查询出库单
        List<OutboundOrder> outboundList = outboundOrderService.list(q -> q.between(OutboundOrder::getOutboundTime, beginDate, endDate));
        Map<String, List<OutboundOrder>> outboundMap = outboundList.stream().collect(
                Collectors.groupingBy(OutboundOrder::getOrderWlnCode));

        // 查询今日订单
        Set<Long> orderIds = new HashSet<>(taskMap.keySet());
        List<OrderInfo> orderInfoList = new ArrayList<>();
        if (!orderIds.isEmpty()) {
            orderInfoList = orderService.list(q -> q.eq(OrderInfo::getSource, 2)
                    .in(BaseIdPo::getId, orderIds)
                    .in(OrderInfo::getDepartmentId, departmentIds));
        }

        Map<String, OrderInfo> orderInfoMap = orderInfoList.stream().collect(Collectors.toMap(OrderInfo::getWlnCode, Function.identity()));

        Set<String> wlnCodeSet = new HashSet<>(outboundMap.keySet());
        wlnCodeSet.addAll(orderInfoMap.keySet());
        if (wlnCodeSet.isEmpty()) {
            return Collections.emptyList();
        }

        // 万里牛sku信息
        Set<String> skuSpecCodes = outboundList.stream().map(OutboundOrder::getSkuSpecCode).collect(Collectors.toSet());
        Map<String, Long> outboundSkuSpecMap;
        if (!skuSpecCodes.isEmpty()) {
            outboundSkuSpecMap = skuSpecService.mapKV(SkuSpec::getCode, BaseIdPo::getId, q -> q.in(SkuSpec::getCode, skuSpecCodes));
        } else {
            outboundSkuSpecMap = new HashMap<>();
        }

        // 查询所有sku规格
        Set<Long> skuSpecIds = taskMap.values().stream().flatMap(Collection::stream).map(ProductionWorkOrder::getSkuSpecId).collect(Collectors.toSet());
        skuSpecIds.addAll(outboundSkuSpecMap.values());
        Map<Long, SkuSpec> skuSpecMap = skuSpecService.byIdsToMap(skuSpecIds);
        // 查询sku关联bom
        Set<Long> bomSpecIds = skuSpecMap.values().stream().map(SkuSpec::getBomSpecId).collect(Collectors.toSet());
        Map<Long, BomSpecBo> bomSpecMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        for (String wlnCode : wlnCodeSet) {
            // 获取出库单的订单sku
            List<OutboundOrder> outboundOrderList = outboundMap.getOrDefault(wlnCode, Collections.emptyList());
            Map<Long, BigDecimal> outboundSkuMap = outboundOrderList.stream().collect(Collectors.toMap(item -> outboundSkuSpecMap.get(item.getSkuSpecCode()), OutboundOrder::getQuantity, BigDecimal::add));

            // 获取生产sku
            OrderInfo orderInfo = orderInfoMap.get(wlnCode);
            List<ProductionWorkOrder> productionWorkOrderList;
            if (orderInfo == null) {
                productionWorkOrderList = Collections.emptyList();
            } else {
                productionWorkOrderList = taskMap.getOrDefault(orderInfo.getId(), Collections.emptyList());
            }
            Map<Long, BigDecimal> productionSkuMap = productionWorkOrderList.stream().collect(Collectors.toMap(ProductionWorkOrder::getSkuSpecId, item -> BigDecimal.ONE, BigDecimal::add));

            Set<Long> skuSpecIdSet = new HashSet<>(outboundSkuMap.keySet());
            skuSpecIdSet.addAll(productionSkuMap.keySet());

            if (skuSpecIdSet.isEmpty()) {
                continue;
            }

            for (Long skuSpecId : skuSpecIdSet) {
                Long key = null;
                SkuSpec skuSpec = skuSpecMap.get(skuSpecId);
                if (skuSpec != null) {
                    key = skuSpec.getBomSpecId();
                }
                BomSpecBo bomSpecBo = bomSpecMap.get(key);
                if (bomSpecBo == null) {
                    key = null;
                } else if (!Objects.equals(bomSpecBo.getClassifyParentId(), 1L)) {
                    continue;
                }

                OrderBomDifferenceVo bomDifferenceVo = differenceMap.computeIfAbsent(key, item -> {
                    OrderBomDifferenceVo tempOrderBomDifferenceVo = new OrderBomDifferenceVo();
                    tempOrderBomDifferenceVo.setBomSpecCode(bomSpecBo == null ? "未绑定" : bomSpecBo.getBomSpecCode());
                    tempOrderBomDifferenceVo.setMesQuantity(BigDecimal.ZERO);
                    tempOrderBomDifferenceVo.setWlnQuantity(BigDecimal.ZERO);
                    tempOrderBomDifferenceVo.setOrderWlnCodes(new HashSet<>());
                    return tempOrderBomDifferenceVo;
                });
                BigDecimal productionQuantity = productionSkuMap.getOrDefault(skuSpecId, BigDecimal.ZERO);
                BigDecimal outboundQuantity = outboundSkuMap.getOrDefault(skuSpecId, BigDecimal.ZERO);
                bomDifferenceVo.setMesQuantity(bomDifferenceVo.getMesQuantity().add(productionQuantity));
                bomDifferenceVo.setWlnQuantity(bomDifferenceVo.getWlnQuantity().add(outboundQuantity));
                if (!ObjectUtil.equals(productionQuantity, outboundQuantity)) {
                    // 新增差异订单号
                    bomDifferenceVo.getOrderWlnCodes().add(wlnCode);
                }
                differenceMap.put(key, bomDifferenceVo);
            }
        }
        // 筛选出有差异的bom数据
        return differenceMap.values().stream().filter(item -> !item.getOrderWlnCodes().isEmpty()).collect(Collectors.toList());
    }

    @Override
    public WlnSalesOutStorageVo getWlnSalesOutStorageInfo(DailyBoardSelectDto dto) {
        WlnSalesOutStorageVo vo = new WlnSalesOutStorageVo();
        vo.setSumBomCount(BigDecimal.ZERO);
        vo.setSumOrderCount(0);
        vo.setBzSumBomCount(BigDecimal.ZERO);
        vo.setBzSumOrderCount(0);
        vo.setSySumBomCount(BigDecimal.ZERO);
        vo.setSySumOrderCount(0);
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());

        // 查询今日出库单出库数据
        List<OutboundOrder> outboundOrderMap = outboundOrderService.list(q -> q
                .between(OutboundOrder::getOutboundTime, beginDate, endDate));
        if (ObjectUtil.isEmpty(outboundOrderMap)) {
            return vo;
        }

        Set<String> skuSpecCodes = outboundOrderMap.stream().map(OutboundOrder::getSkuSpecCode).collect(Collectors.toSet());
        Map<String, SkuSpec> skuSpecMap = skuSpecService.mapKEntity(SkuSpec::getCode, q -> q.in(SkuSpec::getCode, skuSpecCodes));
        List<Long> bomSpecIds = skuSpecMap.values().stream().map(SkuSpec::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        List<DailyBoardOutStorageBomVo> outStorageBomVoList = new ArrayList<>();
        List<DailyBoardOutStorageBomVo> bzOutStorageBomVoList = new ArrayList<>();
        List<DailyBoardOutStorageBomVo> syOutStorageBomVoList = new ArrayList<>();
        Set<String> bzOrderCodes = new HashSet<>();
        Set<String> syOrderCodes = new HashSet<>();
        for (OutboundOrder outboundOrder : outboundOrderMap) {
            SkuSpec skuSpec = skuSpecMap.get(outboundOrder.getSkuSpecCode());
            BomSpecBo bomSpec = null;
            if (skuSpec != null) {
                bomSpec = bomSpecMap.get(skuSpec.getBomSpecId());
                if (bomSpec != null && !Objects.equals(bomSpec.getClassifyParentId(), 1L)) {
                    continue;
                }
            }
            // 判断是否是佰卓订单
            boolean flag = Objects.equals(outboundOrder.getStorageCode(), "T007");
            if (flag) {
                bzOrderCodes.add(outboundOrder.getOrderWlnCode());
            } else {
                syOrderCodes.add(outboundOrder.getOrderWlnCode());
            }
            DailyBoardOutStorageBomVo bomVo = new DailyBoardOutStorageBomVo();
            if (bomSpec != null) {
                bomVo.setBomSpecId(bomSpec.getBomSpecId());
                bomVo.setBomSpecCode(bomSpec.getBomSpecCode());
                bomVo.setBomSpecName(bomSpec.getBomSpecName());
            }
            bomVo.setQuantity(outboundOrder.getQuantity());
            outStorageBomVoList.add(bomVo);
            if (Objects.equals(outboundOrder.getStorageCode(), "T007")) {
                bzOutStorageBomVoList.add(bomVo);
            } else {
                syOutStorageBomVoList.add(bomVo);
            }
        }
        // 合并bom数据
        Map<Long, DailyBoardOutStorageBomVo> outStorageBomMap = outStorageBomVoList.stream().collect(
                Collectors.toMap(
                        DailyBoardOutStorageBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                            BeanUtil.copyProperties(v1, tempVo);
                            tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return tempVo;
                        }));

        Map<Long, DailyBoardOutStorageBomVo> bzOutStorageBomMap = bzOutStorageBomVoList.stream().collect(
                Collectors.toMap(
                        DailyBoardOutStorageBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                            BeanUtil.copyProperties(v1, tempVo);
                            tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return tempVo;
                        }));

        Map<Long, DailyBoardOutStorageBomVo> syOutStorageBomMap = syOutStorageBomVoList.stream().collect(
                Collectors.toMap(
                        DailyBoardOutStorageBomVo::getBomSpecId,
                        Function.identity(),
                        (v1, v2) -> {
                            DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                            BeanUtil.copyProperties(v1, tempVo);
                            tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            return tempVo;
                        }));

        vo.setBzSumOrderCount(bzOrderCodes.size());
        vo.setSySumOrderCount(syOrderCodes.size());
        vo.setSumOrderCount(vo.getBzSumOrderCount() + vo.getSySumOrderCount());
        vo.setSumBomCount(outStorageBomVoList.stream().map(DailyBoardOutStorageBomVo::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add));
        vo.setBzSumBomCount(bzOutStorageBomVoList.stream().map(DailyBoardOutStorageBomVo::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add));
        vo.setSySumBomCount(syOutStorageBomVoList.stream().map(DailyBoardOutStorageBomVo::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add));
        vo.setOutStorageBomList(new ArrayList<>(outStorageBomMap.values()));
        vo.setBzOutStorageBomList(new ArrayList<>(bzOutStorageBomMap.values()));
        vo.setSyOutStorageBomList(new ArrayList<>(syOutStorageBomMap.values()));
        return vo;
    }

    @Override
    public MesProductionOutStorageVo getMesProductionOutStorageInfo(DailyBoardSelectDto dto) {
        MesProductionOutStorageVo vo = new MesProductionOutStorageVo();
        vo.setSumBomCount(BigDecimal.ZERO);
        vo.setSumOrderCount(0);
        vo.setBzSumBomCount(BigDecimal.ZERO);
        vo.setBzSumOrderCount(0);
        vo.setSySumBomCount(BigDecimal.ZERO);
        vo.setSySumOrderCount(0);
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());
        // 查询佰卓和实业的事业部id
        Department department = departmentService.getOne(q -> q.eq(Department::getName, "佰卓").select(BaseIdPo::getId));
        List<Department> departmentList = departmentService.list(q -> q.like(Department::getName, "实业").select(BaseIdPo::getId));
        List<Long> departmentIds = departmentList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        departmentIds.add(department.getId());

        // 查询今日出库单出库数据
        Map<Long, List<ProductionWorkOrder>> taskMap = productionWorkOrderService.mapKGroup(ProductionWorkOrder::getOrderId, q -> q.between(ProductionWorkOrder::getCompleteTime, beginDate, endDate));
        if (ObjectUtil.isEmpty(taskMap)) {
            return vo;
        }
        // 出库单相关的订单数据
        List<OrderInfo> orderList = orderService.list(q -> q
                .eq(OrderInfo::getSource, 2)
                .in(OrderInfo::getId, taskMap.keySet())
                .in(OrderInfo::getDepartmentId, departmentIds));
        if (ObjectUtil.isEmpty(orderList)) {
            return vo;
        }
        // 获取订单sku列表
        List<Long> orderIds = orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIds));
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        // 获取今日订单bom列表
        List<Long> bomSpecIds = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        Map<Long, DailyBoardOutStorageBomVo> outStorageBomMap = new HashMap<>();
        Map<Long, DailyBoardOutStorageBomVo> bzOutStorageBomMap = new HashMap<>();
        Map<Long, DailyBoardOutStorageBomVo> syOutStorageBomMap = new HashMap<>();
        int bzCount = 0;
        int syCount = 0;
        for (OrderInfo orderInfo : orderList) {
            BigDecimal quantity = BigDecimal.ZERO;
            // 当前订单出库数据
            List<ProductionWorkOrder> productionWorkOrderList = taskMap.get(orderInfo.getId());
            // 出库单中主材bom
            List<DailyBoardOutStorageBomVo> outStorageBomVoList = new ArrayList<>();
            for (ProductionWorkOrder productionWorkOrder : productionWorkOrderList) {
                OrderSku orderSku = orderSkuMap.get(productionWorkOrder.getOrderSkuId());
                if (orderSku == null) {
                    continue;
                }
                BomSpecBo bomSpecBo = bomSpecBoMap.get(orderSku.getBomSpecId());
                if (bomSpecBo == null) {
                    quantity = quantity.add(BigDecimal.ONE);
                    continue;
                }
                if (Objects.equals(bomSpecBo.getClassifyParentId(), 1L)) {
                    DailyBoardOutStorageBomVo bomVo = new DailyBoardOutStorageBomVo();
                    bomVo.setBomSpecId(bomSpecBo.getBomSpecId());
                    bomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
                    bomVo.setBomSpecName(bomSpecBo.getBomSpecName());
                    bomVo.setQuantity(BigDecimal.ONE);
                    outStorageBomVoList.add(bomVo);
                    quantity = quantity.add(BigDecimal.ONE);
                }
            }
            if (ObjectUtil.equals(quantity, BigDecimal.ZERO)) {
                continue;
            }

            // 保存bom数量
            vo.setSumBomCount(vo.getSumBomCount().add(quantity));

            outStorageBomMap = Stream.concat(outStorageBomMap.values().stream(), outStorageBomVoList.stream())
                    .collect(Collectors.toMap(
                            DailyBoardOutStorageBomVo::getBomSpecId,
                            Function.identity(),
                            (v1, v2) -> {
                                DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                                BeanUtil.copyProperties(v1, tempVo);
                                tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                return tempVo;
                            }
                    ));
            // 佰卓订单
            if (ObjectUtil.equals(orderInfo.getDepartmentId(), department.getId())) {
                bzCount++;
                vo.setBzSumOrderCount(bzCount);
                bzOutStorageBomMap = Stream.concat(bzOutStorageBomMap.values().stream(), outStorageBomVoList.stream())
                        .collect(Collectors.toMap(
                                DailyBoardOutStorageBomVo::getBomSpecId,
                                Function.identity(),
                                (v1, v2) -> {
                                    DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                                    BeanUtil.copyProperties(v1, tempVo);
                                    tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                    return tempVo;
                                }
                        ));
                vo.setBzSumBomCount(vo.getBzSumBomCount().add(quantity));
            } else {
                syCount++;
                vo.setSySumOrderCount(syCount);
                syOutStorageBomMap = Stream.concat(syOutStorageBomMap.values().stream(), outStorageBomVoList.stream())
                        .collect(Collectors.toMap(
                                DailyBoardOutStorageBomVo::getBomSpecId,
                                Function.identity(),
                                (v1, v2) -> {
                                    DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                                    BeanUtil.copyProperties(v1, tempVo);
                                    tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                    return tempVo;
                                }
                        ));
                vo.setSySumBomCount(vo.getSySumBomCount().add(quantity));
            }
        }
        vo.setSumOrderCount(vo.getBzSumOrderCount() + vo.getSySumOrderCount());
        vo.setOutStorageBomList(new ArrayList<>(outStorageBomMap.values()));
        vo.setBzOutStorageBomList(new ArrayList<>(bzOutStorageBomMap.values()));
        vo.setSyOutStorageBomList(new ArrayList<>(syOutStorageBomMap.values()));
        return vo;
    }

    @Override
    public SelfOrderOutStorageVo getSelfOrderOutStorageInfo(DailyBoardSelectDto dto) {
        SelfOrderOutStorageVo vo = new SelfOrderOutStorageVo();
        vo.setSumBomCount(BigDecimal.ZERO);
        vo.setSumOrderCount(0);
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());
        // 查询今日出库单出库数据
        Map<Long, List<ProductionWorkOrder>> taskMap = productionWorkOrderService.mapKGroup(ProductionWorkOrder::getOrderId, q -> q.between(ProductionWorkOrder::getCompleteTime, beginDate, endDate));
        if (ObjectUtil.isEmpty(taskMap)) {
            return vo;
        }
        // 出库单相关的订单数据
        List<OrderInfo> orderList = orderService.list(q -> q
                .eq(OrderInfo::getSource, 1)
                .eq(OrderInfo::getType, 1)
                .in(OrderInfo::getId, taskMap.keySet()));
        if (ObjectUtil.isEmpty(orderList)) {
            return vo;
        }
        vo.setSumOrderCount(orderList.size());
        // 获取订单sku列表
        List<Long> orderIds = orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, orderIds));
        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        // 获取今日订单bom列表
        List<Long> bomSpecIds = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIds);

        Map<Long, DailyBoardOutStorageBomVo> outStorageBomMap = new HashMap<>();
        for (OrderInfo orderInfo : orderList) {
            BigDecimal quantity = BigDecimal.ZERO;
            // 当前订单出库数据
            List<ProductionWorkOrder> productionWorkOrderList = taskMap.get(orderInfo.getId());
            // 出库单中主材bom
            List<DailyBoardOutStorageBomVo> outStorageBomVoList = new ArrayList<>();
            for (ProductionWorkOrder productionWorkOrder : productionWorkOrderList) {
                OrderSku orderSku = orderSkuMap.get(productionWorkOrder.getOrderSkuId());
                if (orderSku == null) {
                    continue;
                }
                Long bomSpecId = orderSku.getBomSpecId();
                BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
                if (Objects.equals(bomSpecBo.getClassifyParentId(), 1L) && Objects.equals(productionWorkOrder.getStatus(), 3)) {
                    DailyBoardOutStorageBomVo bomVo = new DailyBoardOutStorageBomVo();
                    bomVo.setBomSpecId(bomSpecBo.getBomSpecId());
                    bomVo.setBomSpecCode(bomSpecBo.getBomSpecCode());
                    bomVo.setBomSpecName(bomSpecBo.getBomSpecName());
                    bomVo.setQuantity(new BigDecimal(1));
                    outStorageBomVoList.add(bomVo);
                    quantity = quantity.add(new BigDecimal(1));
                }
            }
            // 保存bom数量
            vo.setSumBomCount(vo.getSumBomCount().add(quantity));
            outStorageBomMap = Stream.concat(outStorageBomMap.values().stream(), outStorageBomVoList.stream())
                    .collect(Collectors.toMap(
                            DailyBoardOutStorageBomVo::getBomSpecId,
                            Function.identity(),
                            (v1, v2) -> {
                                DailyBoardOutStorageBomVo tempVo = new DailyBoardOutStorageBomVo();
                                BeanUtil.copyProperties(v1, tempVo);
                                tempVo.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                                return tempVo;
                            }
                    ));
        }
        vo.setOutStorageBomList(new ArrayList<>(outStorageBomMap.values()));
        return vo;
    }

    @Override
    public void checkStatementOfAccount(DailyBoardSelectDto dto) {
        Date beginDate = DateUtil.beginOfDay(dto.getBeginDate());
        Date endDate = DateUtil.endOfDay(dto.getEndDate());
        // 查询佰卓和实业的事业部id
        List<Department> departmentList = departmentService.list(q -> q.like(Department::getName, "实业").or().eq(Department::getName, "佰卓").select(BaseIdPo::getId));
        List<Long> departmentIds = departmentList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        // 查询今日生成完成任务
        List<ProductionWorkOrder> productionWorkOrderList = productionWorkOrderService.list(q -> q.between(ProductionWorkOrder::getCompleteTime, beginDate, endDate));
        long productionOrderCount;
        if (productionWorkOrderList.isEmpty()) {
            productionOrderCount = 0L;
        } else {
            List<Long> orderIds = productionWorkOrderList.stream().map(ProductionWorkOrder::getOrderId).collect(Collectors.toList());
            productionOrderCount = orderService.count(q -> q.in(OrderInfo::getDepartmentId, departmentIds).in(BaseIdPo::getId, orderIds));
        }

        // 查询今日出库单出库数据
        List<OutboundOrder> outboundOrderList = outboundOrderService.list(q -> q.between(OutboundOrder::getOutboundTime, beginDate, endDate));
        long wlnOrderCount;
        if (outboundOrderList.isEmpty()) {
            wlnOrderCount = 0L;
        } else {
            List<String> wlnCodes = outboundOrderList.stream().map(OutboundOrder::getOrderWlnCode).collect(Collectors.toList());
            wlnOrderCount = orderService.count(q -> q.in(OrderInfo::getDepartmentId, departmentIds).in(OrderInfo::getWlnCode, wlnCodes));
        }
        if (ObjectUtil.notEqual(productionOrderCount, wlnOrderCount)) {
            throw new ServiceException("核对失败，订单总数存在差异!");
        }

        // 查询bom总数
        List<ProductionWorkOrder> workOrderList = productionWorkOrderList.stream().filter(item -> Objects.equals(item.getStatus(), 3)).collect(Collectors.toList());
        BigDecimal productionBomQuantity = new BigDecimal(workOrderList.size());
        BigDecimal outboundBomQuantity = outboundOrderList.stream().map(OutboundOrder::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);

        if (ObjectUtil.notEqual(productionBomQuantity, outboundBomQuantity)) {
            throw new ServiceException("核对失败，bom出库总数存在差异!");
        }

        // 更新对账单核对状态
        statementOfAccountService.update(q -> q
                .set(StatementOfAccount::getCheckStatus, StatusConstant.YES)
                .in(StatementOfAccount::getDepartmentId, departmentIds)
                .between(StatementOfAccount::getTimePeriod, beginDate, endDate));
    }
}
