package com.sd.business.service.board.impl;

import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.board.dto.InOutStorageBoardSelectDto;
import com.sd.business.entity.board.dto.SalesBoardSelectDto;
import com.sd.business.entity.board.vo.HazardousInventoryVo;
import com.sd.business.entity.board.vo.InOutStorageBoardVo;
import com.sd.business.entity.board.vo.InTransitBomVo;
import com.sd.business.entity.board.vo.InventoryTurnoverRateVo;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.board.InventoryBoardMapper;
import com.sd.business.service.board.InventoryBoardService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;

/**
 * <p>
 * 库存看板 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-16
 */
@Service
public class InventoryBoardServiceImpl implements InventoryBoardService {

    @Resource
    private InventoryBoardMapper inventoryBoardMapper;

    @Override
    public Page<HazardousInventoryVo> getHazardousInventoryPage(SalesBoardSelectDto dto) {
        IWrapper<HazardousInventoryVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.orderByDesc("if(saleQuantity > 0, 1, 0)");
        wrapper.orderByDesc("if(inventoryQuantity > 0, 1, 0)");
        wrapper.orderByAsc("predictOutStorageDays");
        return inventoryBoardMapper.getHazardousInventoryPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<InTransitBomVo> getInTransitBomPage(SalesBoardSelectDto dto) {
        IWrapper<InTransitBomVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.orderByAsc("nextDeliveryDays");
        wrapper.orderByDesc("inTransitQuantity");
        wrapper.groupBy("bs.id");
        return inventoryBoardMapper.getInTransitBomPage(dto.getPage(), wrapper);
    }

    @Override
    public Page<InventoryTurnoverRateVo> getTurnoverRatePage(SalesBoardSelectDto dto) {
        IWrapper<InventoryTurnoverRateVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        if (Objects.equals(dto.getSortBy(), 1)) {
            wrapper.orderByAsc("turnoverRate");
        } else {
            wrapper.orderByDesc("turnoverRate");
        }
        long days = 30;
        if (dto.getBeginTime() != null && dto.getEndTime() != null) {
            days = DateUtil.betweenDay(dto.getBeginTime(), dto.getEndTime(), true) + 1;
        }
        return inventoryBoardMapper.getTurnoverRatePage(dto.getPage(), wrapper, days);
    }

    @Override
    public Page<InOutStorageBoardVo> getInOutStoragePage(InOutStorageBoardSelectDto dto) {
        IWrapper<InOutStorageBoardVo> wrapper = IWrapper.getWrapper();
        wrapper.eq("bc", BomClassify::getParentId, 1);
        wrapper.eq("b", Bom::getItemSubclass, dto.getItemSubclass());
        wrapper.eq("b", Bom::getChromatophore, dto.getChromatophore());
        wrapper.eq("bs", BomSpec::getWidth, dto.getBomSpecWidth());
        wrapper.eq("bs", BomSpec::getHeight, dto.getBomSpecHeight());
        wrapper.eq("bs", BomSpec::getColour, dto.getColour());
        wrapper.eq("ios", InOutStorage::getType, dto.getType());
        wrapper.eq("ios", InOutStorage::getDetailType, InDetailTypeEnum.PURCHASE.getKey());
        wrapper.eq("ios", InOutStorage::getWarehouseId, WarehouseConstant.SEMI_FINISHED_PRODUCT);
        wrapper.eq("ios", InOutStorage::getDepartmentId, DepartmentConstant.SD_SPORTS);
        wrapper.ge("ios", InOutStorage::getInOutStorageTime, dto.getBeginTime());
        wrapper.le("ios", InOutStorage::getInOutStorageTime, dto.getEndTime());
        wrapper.groupBy("bs.id");
        if (Objects.equals(dto.getSortBy(), 1)) {
            wrapper.orderByAsc("quantity");
        } else {
            wrapper.orderByDesc("quantity");
        }
        return inventoryBoardMapper.getInOutStoragePage(dto.getPage(), wrapper);
    }
}
